package appia.protocols.fifo;

//////////////////////////////////////////////////////////////////////
//                                                                  //
// Appia: protocol development and composition framework             //
//                                                                  //
// Version: 1.0/J                                                   //
//                                                                  //
// Copyright, 2000, Universidade de Lisboa                          //
// All rights reserved                                              //
// See license.txt for further information                          //
//                                                                  //
// Class: AckEvent: Acknowledgment event                            //
//                                                                  //
// Author: Hugo Miranda, 05/2000                                    //
//                                                                  //
// Change Log:                                                      //
// Data, alteracao, razao/consequncias                             //
//                                                                  //
//////////////////////////////////////////////////////////////////////

import appia.*;
import appia.events.*;

/**
 * Class AckEvent extends SendableEvent and is intended to send
 * acknowledgments of messages. The constructor simplifies the operation
 * by coping the necessary attributes from another message
 *
 * @author Hugo Miranda
 * @see    FifoLayer
 * @see    Event
 */

public class AckEvent extends SendableEvent {
    /** 
     * This constructor creates an acknowledge based on a model
     * event. dest and source fields are switched, direction is
     * inverted. channel is copied from the model and generator is
     * obtained from the session pointer.
     */

    public AckEvent(SendAckTimer e,Session gen) throws AppiaEventException {
	/* Calls the main event Constructor Class */
	super();
	setChannel(e.getChannel());
	Direction d=new Direction(e.getDirection().direction);
	d.invert();
	setDirection(d);
	setSource(gen);
	dest=e.peer.peer;
	source=e.source;
	init();
    }

    public AckEvent() {}
}
