package appia.protocols.fifo;

//////////////////////////////////////////////////////////////////////
//                                                                  //
// Appia: protocol development and composition framework            //
//                                                                  //
// Version: 1.0/J                                                   //
//                                                                  //
// Copyright, 2000, Universidade de Lisboa                          //
// All rights reserved                                              //
// See license.txt for further information                          //
//                                                                  //
// Class: PeerInfo: Fifo ordering support class grouping            //
// information for a certain peer                                   //
//                                                                  //
// Author: Hugo Miranda, 05/2000                                    //
//                                                                  //
// Change Log:                                                      //
// Data, alteracao, razao/consequncias                             //
//                                                                  //
//////////////////////////////////////////////////////////////////////

import appia.events.*;
import appia.*;

/**
 * PeerInfo contains next expected sequence number, next sequence
 * number to send and out of order messages for a single peer of a
 * fifo session
 */

class PeerInfo {
    public  int              nextOutgoing,firstUnconfirmed,
	                     nextIncoming,lastAckSent;
    private WaitingMessage[] waitQueue;
    public  Object           peer;

    private int              queueSize=11;

    /* Used to now if FIFO has give up sending messages to this endpoint.
       If so, the instance won't be at the Session peer vector.
    */
    public  boolean          failed=false;

    public PeerInfo(Object peer) {
	this.peer=peer;
	nextOutgoing=0;
	nextIncoming=0;
	firstUnconfirmed=0;
	lastAckSent=-1;
	waitQueue=new WaitingMessage[queueSize];
    }

    public void incOutgoing() {
	nextOutgoing++;
    }

    public void incIncoming() {
	nextIncoming++;
    }

    public boolean isNext(int check) {
	return nextIncoming==check;
    }

    public boolean isDuplicated(int check) {
	return nextIncoming>check;
    }

    /* Queue handling is made based on mod nextIncoming operations */

    SendableEvent dequeueNextIncoming() {

	int queueIndex=nextIncoming % queueSize;

	if(waitQueue[queueIndex]==null)
	    return null;

	SendableEvent ret=waitQueue[queueIndex].e;
	waitQueue[queueIndex]=null;

	return ret;
    }

    void enqueueIncoming(SendableEvent e,int seqNumber) {

	/* Messages too far from next expected will be thrown away */
	if(seqNumber-nextIncoming>queueSize) {
	    return;
	}
	int queueIndex=seqNumber % queueSize;
	if(waitQueue[queueIndex]==null) {
	    waitQueue[queueIndex]=new WaitingMessage(e,seqNumber);
	}
    }

    void confirmedUntil(int seq) {
	/* The if clause prevents out of order acks and performs a 
	   sanity check: acknowledgments of unsent messages */
	if(firstUnconfirmed<seq && seq<=nextOutgoing)
	    firstUnconfirmed=seq;
    }

    WaitingMessage addUnconfirmed(SendableEvent e,int seqNumber) {
	WaitingMessage we=new WaitingMessage(e,seqNumber);
	return we;
    }
    /* Since this layer uses cumulative acks, checking the first 
       unacknowledged suffices */
    boolean isAcknowledged(WaitingMessage we) {
	return firstUnconfirmed>we.seqNumber;
    }

    public int getWaitingQueueSize() {
	return -1;
    }
}
