
/**
 * Title:        Apia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia.protocols.group;

import appia.*;


public class StableLayer extends Layer {

  public static final long GOSSIP_TIME=1000; // time between stable gossips in milliseconds

  public StableLayer() {
    try {
      Class stable=Class.forName("appia.protocols.group.events.Stable");
      Class gossip=Class.forName("appia.protocols.group.events.StableGossip");
      Class stabilize=Class.forName("appia.protocols.group.events.Stabilize");
      Class blockok=Class.forName("appia.protocols.group.events.BlockOk");
      Class view=Class.forName("appia.protocols.group.events.View");
      Class group=Class.forName("appia.protocols.group.events.GroupSendableEvent");
      Class periodic=Class.forName("appia.events.channel.PeriodicTimer");
      Class retrans=Class.forName("appia.protocols.group.events.Retransmission");
      Class fail=Class.forName("appia.protocols.group.events.Fail");
      Class debug=Class.forName("appia.events.channel.Debug");

      evProvide=new Class[3];
      evProvide[0]=stable;
      evProvide[1]=gossip;
      evProvide[2]=retrans;

      evRequire=new Class[2];
      evRequire[0]=view;
      evRequire[1]=periodic;

      evAccept=new Class[9];
      evAccept[0]=gossip;
      evAccept[1]=stabilize;
      evAccept[2]=blockok;
      evAccept[3]=view;
      evAccept[4]=group;
      evAccept[5]=periodic;
      evAccept[6]=retrans;
      evAccept[7]=fail;
      evAccept[8]=debug;

    } catch (ClassNotFoundException ex) {
      throw new AppiaGroupError("StableLayer:"+ex.getMessage());
    }
  }

  public Session createSession() {
    return new StableSession(this,GOSSIP_TIME);
  }
}