
/**
 * Title:        Apia<p>
 * Description:  Protocol development and composition framework<p>
 * Copyright:    Copyright (c) Alexandre Pinto & Hugo Miranda & Luis Rodrigues<p>
 * Company:      F.C.U.L.<p>
 * @author Alexandre Pinto & Hugo Miranda & Luis Rodrigues
 * @version 1.0
 */
package appia.protocols.group;

import appia.*;
import appia.protocols.group.events.*;
import appia.events.channel.EchoEvent;
import appia.message.*;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;

public class SyncSession extends Session {

  public SyncSession(Layer layer) {
    super(layer);
  }

  private boolean block=false;
  private boolean[] blocked;
  private ViewState vs;
  private LocalState ls;

  public void handle(Event event) {

    // Block
    if (event instanceof Block) {

      if (event.getDirection().direction==Direction.DOWN) {
        block=true;

        if (blocked == null) {
          try { event.go(); } catch (AppiaEventException ex) { ex.printStackTrace(); }

          begin(event.getChannel());
        } else {
          if (all_blocked())
             end(event.getChannel());
        }
      } else {
        if (blocked == null)
           begin(event.getChannel());
      }

      return;
    }

    // BlockOk
    if (event instanceof BlockOk) {

      if (blocked == null) {
        begin(event.getChannel());
      }

      ok((BlockOk)event);
      return;
    }

    // Fail
    if (event instanceof Fail) {
      if (block && all_blocked()) {
        end(event.getChannel());
      }
    }

    // View
    if (event instanceof View) {
      View ev=(View)event;

      vs=ev.vs;
      ls=ev.ls;

      block=false;
      blocked=null;

      try { ev.go(); } catch (AppiaEventException ex) { ex.printStackTrace(); }
      return;
    }
  }

  private void begin(Channel channel) {
    try {
      BlockOk ev=new BlockOk();
      ev.group=vs.group;
      ev.view_id=vs.id;
      EchoEvent echo=new EchoEvent(ev,channel,new Direction(Direction.UP),this);

      blocked=new boolean[vs.view.length];
      for (int i=0 ; i < blocked.length ; i++) blocked[i]=false;

      echo.go();
    } catch (AppiaEventException ex) {
      ex.printStackTrace();
      blocked=null;
    }
  }

  private void ok(BlockOk ev) {
    if (ev.getDirection().direction == Direction.DOWN) {
      blocked[ls.my_rank]=true;

      serializeBlocked(ev.getObjectsMessage());

      try {
        ev.go();
      } catch (AppiaEventException ex) {
        // no way to recover
        throw new AppiaGroupError(ex.getMessage());
      }

    } else {
      deserializeBlocked(ev.getObjectsMessage());
    }

    if (block && all_blocked()) {
      end(ev.getChannel());
    }
  }

  private void end(Channel channel) {
    try {
      BlockOk ev=new BlockOk(channel,new Direction(Direction.UP),this,vs.group,vs.id);
      ev.go();
    } catch (AppiaEventException ex) {
      // no way to recover
      throw new AppiaGroupError(ex.getMessage());
    }
  }

  private boolean all_blocked() {
    int i;
    boolean result=true;

    for (i=0 ; (i < ls.failed.length) && result ; i++) {
      if (!blocked[i] && ls.failed[i]) {
        blocked[i]=true;
      }

      result=blocked[i];
    }

    return result;
  }

  private void serializeBlocked(ObjectsMessage omsg) {
    omsg.push(blocked.clone());
  }

  private void deserializeBlocked(ObjectsMessage omsg) {
    boolean[] remote_blocked=(boolean[])omsg.pop();

    int i;
    for(i=0 ; i < blocked.length ; i++) {
      blocked[i]= (blocked[i] || remote_blocked[i]);
    }
  }
}